<?php 
namespace Concrete\Package\CcOptinFree\Controllers;

use Loader;
use View;
use URL;
use Asset;

use Concrete\Core\Http\ResponseAssetGroup;
use Concrete\Core\Page\Controller\DashboardPageController;

use Concrete\Package\CcOptinFree\CcOptin;
use Concrete\Package\CcOptinFree\Controllers\CcOptinBaseController;

class AjaxView extends View
{
	protected $ccController;
	protected $params;
	protected $viewJs;

	public function __construct(CcOptinController $controller) {
		parent::__construct($controller->viewServerPath);
		$this->ccController = $controller;
	}

	public function render($view, $params = null) {
		$this->params = $params;

		$viewFile = $this->viewPath . DIRECTORY_SEPARATOR . $view;

		if(file_exists($viewFile . '.php'))
			$this->setViewTemplate($viewFile . '.php');
		else
			throw new \Exception('View not found: ' . $viewFile . '.php');

		$html = Loader::helper('html');

		$filename = $viewFile . '.js';
		if(file_exists($filename)) {
			$this->viewJs = $this->viewPath("$view.js");
		}

		$filename = $viewFile . '.css';
		if(file_exists($filename)) {
			$this->ccController->assets->addHeaderAsset($html->css($this->viewPath("$view.css")));
		}

		return parent::render();
	}

	///// Methods 

	public function viewPath($append = '') {
		// TODO: Make portable, can only be located in controllers ATM
		return CcOptin::path('controllers/' . $this->ccController->controllerName . ($append ? "/$append" : ''));
	}

	public function renderPartial($view, $params = null) {
		if(!file_exists($this->viewPath . DIRECTORY_SEPARATOR . $view . '.php'))
			throw new \Exception('Partial view not found: ' . $this->viewPath . DIRECTORY_SEPARATOR . $view . '.php');

		// TODO: Make portable, can only be located in controllers ATM
		View::element('..' . DIRECTORY_SEPARATOR . 'controllers' . DIRECTORY_SEPARATOR . $this->ccController->controllerName . DIRECTORY_SEPARATOR . $view, 
			$params, CC_OPTIN);
	} 

	/////

	public function setupRender() {}

	public function getScopeItems() {
		$merge = array('view' => $this, 'controller' => $this->ccController);
		return $this->params ? array_merge($merge, $this->params) : $merge;
	}

	// Must be overriden to prevent slash being added before a drive letter in windows.
	protected function constructView($path = false) {
		$this->viewPath = $path;
	}

    protected function getAssetsToOutput() {
        $output = $this->ccController->assets->getAssetsToOutput();

        $return = array();
        $noLoad = array('jquery', 'jquery/ui', 'underscore');

        foreach($output as $position => $assets) {
            foreach($assets as $asset) {
                if (method_exists($asset, 'getAssetHandle')) {
                	if(in_array($asset->getAssetHandle(), $noLoad)) continue;

                    $asset->setAssetPosition(Asset::ASSET_POSITION_HEADER);
                    //$asset->setAssetSupportsMinification(false);
                    //$asset->setAssetSupportsCombination(false);
                }
                $return[Asset::ASSET_POSITION_HEADER][] = $asset;
            }
        }
        
        if($this->viewJs) {
        	$asset = Loader::helper('html')->javascript($this->viewJs);
        	$asset->setAssetPosition(Asset::ASSET_POSITION_FOOTER);
	        $return[Asset::ASSET_POSITION_FOOTER][] = $asset;
        }

        return $return;
    }

	protected function onBeforeGetContents() {
		$this->markHeaderAssetPosition();
	}	

	// Hopefully temp fix to https://github.com/concrete5/concrete5-5.7.0/issues/1300
	public function postProcessViewContents($contents) {
		ob_start();
		$this->markFooterAssetPosition();
		$contents .= ob_get_clean();

		return parent::postProcessViewContents($contents);
	}
}

abstract class CcOptinController implements CcOptinIController
{
	const Get = 'GET';
	const Post = 'POST';

	public $viewServerPath;
	public $controllerName;
	public $isAjax;
	public $assets;

	protected $security;
	protected $securityMode;
	protected $view;

	/**
	* First argument should be __FILE__, then a bitflag 0/1/2 for read/write.
	* If set, execute() access will be based upon this mode.
	*/
	public function __construct($controllerFile, $securityMode = 0) {
		$this->controllerName = basename($controllerFile, '.php');
		$this->viewServerPath = dirname($controllerFile) . DIRECTORY_SEPARATOR . $this->controllerName;
		$this->security = new CcOptinSecurity();
		$this->securityMode = $securityMode;
		
		$this->isAjax = !empty($_SERVER['HTTP_X_REQUESTED_WITH']) && strtolower($_SERVER['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest';

		$this->view = new AjaxView($this);
		$this->assets = new ResponseAssetGroup();
	}

	private function testSecurity() {
		if(($this->securityMode & 1) && !$this->security->canRead()) return false;
		if(($this->securityMode & 2) && !$this->security->canWrite()) return false;
		return true;
	}

	public function execute() {
		if(!$this->testSecurity()) {
			CcOptinBaseController::pageForbidden();
			return;
		}

		if(!empty($_REQUEST['ccaction'])) {
			$method = strtolower($_SERVER['REQUEST_METHOD']);
			$action = $method . '_' . $_REQUEST['ccaction'];
			
			if(method_exists($this, $action)) {
				$this->$action($method == 'post' ? $_POST : $_GET);
				return;
			}

			throw new \Exception("No controller method found: " . $action);
		}

		if($this->isGet() && method_exists($this, 'get'))
			$this->get($_GET);
		else if($this->isPost() && method_exists($this, 'post')) 
			$this->post($_POST);
	}

	public function action($queryArray = null) {
		if(!$this->controllerName) throw new \Exception("No controllerName set.");
		$controllerRoute = str_replace('_controller', '', $this->controllerName);

		$query = $queryArray ? '?' . http_build_query($queryArray) : '';

		return URL::to('/cc_optin/' . $controllerRoute) . $query;
	}

	protected function requireAsset($assetType, $assetHandle = false) {
		$this->assets->requireAsset($assetType, $assetHandle);
	}

	// ===== Http method methods =====

	public function method() {
		return $_SERVER['REQUEST_METHOD'];
	}

	public function isPost() {
		return $this->method() == self::Post;
	}

	public function isGet() {
		return $this->method() == self::Get;
	}

	// ===== Response methods =====

	public function path($append = '') {
		return CcOptin::path('controllers/' . $this->controllerName . '/' . $append);
	}

	public function view($viewName, $params = null) {
		if(!$this->viewServerPath) throw new \Exception("No viewPath set.");
		if(!preg_match('#^[a-zA-Z]+[\w/-]*$#', $viewName)) throw new \Exception("Invalid view name: $viewName");

		echo $this->view->render($viewName, $params);
	}

	public function redirect($url) {
		header("Location: " . $url);
	}

	// ===== IController implementation =====

	public function outputJson($data) {
		CcOptinBaseController::outputJson($data);
	}

	public function outputJsonSuccess($text = true, $extraParams = null) {
		CcOptinBaseController::outputJsonSuccess($text, $extraParams);
	}

	public function outputJsonError($text = false, $extraParams = null) {
		CcOptinBaseController::outputJsonError($text, $extraParams);
	}

	public function security() { 
		return $this->security; 
	}

	// ===== Utility =====

	protected function idList($array) {
		return CcOptinBaseController::idList($array);
	}
}
